@@ -6,8 +6,7 @@ from typing import Union
 import logging
 
 import websockets
-import aiohttp
-from .utils import async_get_token, async_get_devices_data
+from .utils import Client
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -23,28 +22,17 @@ class Device():
     '''
     def __init__(
         self,
-        email:str,
-        password:str,
         device_id:Union[str, int],
-        aiohttp_session:aiohttp.ClientSession=None
+        client:Client
     ):
         self._device_id = int(device_id)
         self._name: str = ''
-        self._token: str = None
-        self._email =  email
-        self._password = password
         self._update_callback_func = None
         self._updater_task = None
 
-        self._im_managing_session = not bool(aiohttp_session)
-        self._session = aiohttp.ClientSession() if not aiohttp_session else aiohttp_session
+        self._client: Client = client
 
 
-    def __del__(self):
-        if not self._session.closed and self._im_managing_session:
-            _LOGGER.info('%s: closing session %s', self._name, self._session)
-            asyncio.create_task(self._session.close())
-
     @staticmethod
     def _structure_attrs(attrs: list):
         '''
@@ -56,9 +44,9 @@ class Device():
 
         for attr in attrs:
             name = attr.get('name')
-            last_read_state = attr.get('last_read_state')
+            state = attr.get('state')
 
-            structure[name] = last_read_state
+            structure[name] = state
         return structure
 
 
@@ -73,17 +61,17 @@ class Device():
 
     async def _async_fetch_state(self):
         '''
-        Fetches device information from SmartRent `/resident` page
+        Fetches device information from SmartRent api
 
         Calls ``_fetch_state_helper`` so device can parse out info and update its state.
 
         Calls function passed into ``set_update_callback`` if it exists.
         '''
         _LOGGER.info('%s: Fetching Status res page call...', self._name)
-        data = await async_get_devices_data(self._email, self._password, self._session)
+        data = await self._client.async_get_devices_data()
 
         # Find device id that belongs to me then call _fetch_state_helper
-        for device in data['devices']:
+        for device in data:
             if device['id'] == self._device_id:
                 self._fetch_state_helper(device)
 
@@ -94,20 +82,15 @@ class Device():
                         self._update_callback_func()
 
 
-    async def _async_update_token(self):
-        '''
-        Updates the internal websocket token for the device
-        '''
-        _LOGGER.info('%s: Update Token res page call...', self._name)
-        self._token = await async_get_token(self._email, self._password, self._session)
-
-
     def start_updater(self):
         '''
         Starts running ``update_state`` in the background
         '''
-        _LOGGER.info('%s: Starting updater task', self._name)
-        self._updater_task = asyncio.create_task(self._async_update_state())
+        if not self._updater_task:
+            _LOGGER.info('%s: Starting updater task', self._name)
+            self._updater_task = asyncio.create_task(self._async_update_state())
+        else:
+            _LOGGER.warning('%s: Updater task already running', self._name)
 
 
     def stop_updater(self):
@@ -144,9 +127,7 @@ class Device():
 
         Calls ``_update_parser`` method for device when event is found
         '''
-        await self._async_update_token()
-
-        uri = SMARTRENT_WEBSOCKET_URI.format(self._token)
+        uri = SMARTRENT_WEBSOCKET_URI.format(self._client.token)
 
         async with websockets.connect(uri) as websocket:
             joiner = JOINER_PAYLOAD.format(device_id=self._device_id)
@@ -183,7 +164,7 @@ class Device():
                     _LOGGER.warning('%s: Got excpetion: %s', self._name, exc)
 
                     _LOGGER.info('%s: Getting new token', self._name)
-                    await self._async_update_token()
+                    await self._client.async_refresh_token()
 
                     # Lets fetch device state just to make sure
                     # we didn't miss anything wile the socket was down
@@ -191,7 +172,7 @@ class Device():
                     await self._async_fetch_state()
 
                     _LOGGER.info('%s: Reconnecting to Websocket...', self._name)
-                    uri = SMARTRENT_WEBSOCKET_URI.format(self._token)
+                    uri = SMARTRENT_WEBSOCKET_URI.format(self._client.token)
 
                     websocket = await websockets.connect(uri)
                     _LOGGER.info('%s: Connected!', self._name)
@@ -230,18 +211,17 @@ class Device():
         '''
         _LOGGER.info('sending payload %s', payload)
 
-        joiner = JOINER_PAYLOAD.format(device_id=self._device_id)
-        uri = SMARTRENT_WEBSOCKET_URI.format(self._token)
-
         async def sender(uri: str, payload: str):
             async with websockets.connect(uri) as websocket:
+                joiner = JOINER_PAYLOAD.format(device_id=self._device_id)
+
                 # Join topic given device id
                 await websocket.send(joiner)
                 # Send payload
                 await websocket.send(payload)
 
         try:
-            uri = SMARTRENT_WEBSOCKET_URI.format(self._token)
+            uri = SMARTRENT_WEBSOCKET_URI.format(self._client.token)
 
             await sender(uri, payload)
 
@@ -249,8 +229,8 @@ class Device():
             _LOGGER.debug('Possible issue during send_payload: %s', exc)
 
             # update token once
-            await self._async_update_token()
+            await self._client.async_refresh_token()
 
-            uri = SMARTRENT_WEBSOCKET_URI.format(self._token)
+            uri = SMARTRENT_WEBSOCKET_URI.format(self._client.token)
 
             await sender(uri, payload)
